module Pages.Arguments exposing (Model, Msg, page)

import Config.Colour as T exposing (..)
import Config.Format as O exposing (..)
import Config.Identity as I exposing (..)
import Debate.Arguments.Abortion exposing (..)
import Debate.Arguments.Agnosticism exposing (..)
import Debate.Arguments.AgriculturalPredation exposing (..)
import Debate.Arguments.AnabolicKeto exposing (..)
import Debate.Arguments.AnimalRights exposing (..)
import Debate.Arguments.AntagonisticPleiotropy exposing (..)
import Debate.Arguments.AntiRewilding exposing (..)
import Debate.Arguments.AntiVandalism exposing (..)
import Debate.Arguments.ApoBCVD exposing (..)
import Debate.Arguments.BoobyTrapPagers exposing (..)
import Debate.Arguments.CarbsObesity exposing (..)
import Debate.Arguments.ColonizingNature exposing (..)
import Debate.Arguments.CropDeaths exposing (..)
import Debate.Arguments.DairyCowRape exposing (..)
import Debate.Arguments.DietaryCholesterol exposing (..)
import Debate.Arguments.EfilismPatrolSquad exposing (..)
import Debate.Arguments.EpidemiologyCausality exposing (..)
import Debate.Arguments.EthicalSlurs exposing (..)
import Debate.Arguments.FineTuning exposing (..)
import Debate.Arguments.FlatEarthDebunk exposing (..)
import Debate.Arguments.FructoseNAFLD exposing (..)
import Debate.Arguments.HealthPromotingFoods exposing (..)
import Debate.Arguments.HealthSeeker exposing (..)
import Debate.Arguments.HealthyChocolate exposing (..)
import Debate.Arguments.HealthyDairy exposing (..)
import Debate.Arguments.HealthyFattyFish exposing (..)
import Debate.Arguments.HealthyFibre exposing (..)
import Debate.Arguments.HealthyFood exposing (..)
import Debate.Arguments.HealthyPlantFoods exposing (..)
import Debate.Arguments.HealthySeedOils exposing (..)
import Debate.Arguments.HealthySoy exposing (..)
import Debate.Arguments.ImmortalityReductio exposing (..)
import Debate.Arguments.Malondialdehyde exposing (..)
import Debate.Arguments.OddOrderPredators exposing (..)
import Debate.Arguments.Omega3Omega6Ratio exposing (..)
import Debate.Arguments.Ostroveganism exposing (..)
import Debate.Arguments.PlantBasedCVDReversal exposing (..)
import Debate.Arguments.PollinationReductio exposing (..)
import Debate.Arguments.PolyphenolReductio exposing (..)
import Debate.Arguments.ScratcherPioneers exposing (..)
import Debate.Arguments.SodiumCVD exposing (..)
import Debate.Arguments.TMAOCausality exposing (..)
import Debate.Arguments.Template exposing (argument)
import Debate.Arguments.TransPeople exposing (..)
import Debate.Arguments.TruncatedMeta exposing (..)
import Debate.Arguments.UnhealthyCoconutOil exposing (..)
import Debate.Arguments.UnhealthyProcessedMeat exposing (..)
import Debate.Arguments.UnhealthyRedMeat exposing (..)
import Debate.Arguments.UnhealthySaturatedFat exposing (..)
import Debate.Arguments.VeganSocietyReductio exposing (..)
import Debate.Helpers exposing (..)
import Debate.Types exposing (..)
import Effect exposing (Effect)
import Element exposing (..)
import Element.Border as D exposing (..)
import Element.Font as F
import Html
import Html.Attributes as H exposing (style, title, wrap)
import Layouts
import List.Extra as L exposing (..)
import Page exposing (Page)
import Ports exposing (gotArgHeight)
import Route exposing (Route)
import Shared
import View exposing (View)


page : Shared.Model -> Route () -> Page Model Msg
page shared route =
    Page.new
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view shared
        }
        |> Page.withLayout toLayout


toLayout : Model -> Layouts.Layout Msg
toLayout model =
    Layouts.Navbar {}



-- INIT


type alias Model =
    { argExpansions : List Bool
    , argHeights : List Int
    }


init : () -> ( Model, Effect Msg )
init () =
    let
        numArgs : Int
        numArgs =
            List.length argumentList
    in
    ( { argExpansions = List.repeat numArgs False
      , argHeights = List.repeat numArgs 0
      }
    , Effect.none
    )



-- UPDATE


type Msg
    = GotArgHeight (List Int)
    | ToggleExpandArg Int


update : Msg -> Model -> ( Model, Effect Msg )
update msg model =
    Debug.log (Debug.toString model.argHeights) <|
        case msg of
            GotArgHeight argHeights ->
                ( { model | argHeights = argHeights }, Effect.none )

            ToggleExpandArg index ->
                ( case getAt index model.argExpansions of
                    Nothing ->
                        model

                    Just isExpanded ->
                        { model
                            | argExpansions =
                                setAt index
                                    (not isExpanded)
                                    model.argExpansions
                        }
                , Effect.sendCmd <| Ports.getArgHeight <| List.length model.argHeights
                )


subscriptions : Model -> Sub Msg
subscriptions model =
    gotArgHeight GotArgHeight



-- VIEW


view : Shared.Model -> Model -> View Msg
view shared model =
    { title = debateName
    , attributes = []
    , element = debateContainer shared model
    }


debateContainer : Shared.Model -> Model -> Element Msg
debateContainer shared model =
    let
        a : Device
        a =
            shared.device
    in
    topLevelContainer <| debateList model


debateList : Model -> Element Msg
debateList model =
    let
        argColumn : Bool -> Element Msg
        argColumn isHidden =
            column
                [ padding 30
                , spacing 15
                ]
            <|
                List.map argumentMaker <|
                    List.map4
                        (\w x y z ->
                            { arg = w
                            , index = x
                            , isExpanded = y
                            , maybeHeight =
                                if isHidden then
                                    Nothing

                                else
                                    z
                            , titleClickMsg = ToggleExpandArg x
                            }
                        )
                        argumentList
                        (List.range 0 <| List.length model.argHeights)
                        model.argExpansions
                        (List.map Just model.argHeights)
    in
    el
        [ behindContent <| argColumn True
        , spacing 10
        , centerX
        , centerY
        , alignTop
        , paddingEach { top = 30, bottom = 30, left = 30, right = 30 }
        ]
    <|
        argColumn False


argumentList : List Argument
argumentList =
    [ argumentApoBCVD
    , argumentAnabolicKeto
    , argumentAntagonisticPleiotropy
    , argumentCarbsObesity
    , argumentDietaryCholesterol
    , argumentFructoseNAFLD
    , argumentHealthPromotingFoods
    , argumentHealthSeeker
    , argumentHealthyChocolate
    , argumentHealthyDairy
    , argumentHealthyFattyFish
    , argumentHealthyFibre
    , argumentHealthyFood
    , argumentHealthyPlantFoods
    , argumentHealthySeedOils
    , argumentHealthySoy
    , argumentMalondialdehyde
    , argumentOmega3Omega6Ratio
    , argumentPlantBasedCVDReversal
    , argumentPolyphenolReductio
    , argumentSodiumCVD
    , argumentTMAOCausality
    , argumentUnhealthyCoconutOil
    , argumentUnhealthyProcessedMeat
    , argumentUnhealthyRedMeat
    , argumentUnhealthySaturatedFat
    , argumentAbortion
    , argumentAgnosticism
    , argumentAgriculturalPredation
    , argumentAnimalRights
    , argumentAntiRewilding
    , argumentAntiVandalism
    , argumentColonizingNature
    , argumentCropDeaths
    , argumentDairyCowRape
    , argumentEfilismPatrolSquad
    , argumentEthicalSlurs
    , argumentFineTuning
    , argumentImmortalityReductio
    , argumentOddOrderPredators
    , argumentOstroveganism
    , argumentPollinationReductio
    , argumentScratcherPioneers
    , argumentTransPeople
    , argumentVeganSocietyReductio
    , argumentBoobyTrapPagers
    , argumentEpidemiologyCausality
    , argumentFlatEarthDebunk
    , argumentTruncatedMeta
    ]
